//
//  AccelBump.m
//  AccelBump
//
//  Created by manuMohan on 1/3/13.
//  Copyright (c) 2013 manuMohan. All rights reserved.
//

#import "AccelBump.h"
#import "NetworkManager.h"
#import "BumpManager.h"
#import "LocationManager.h"
#import "UtilsManager.h"
#import "AccelBumpParser.h"

#import "ABConstants.h"

static AccelBump *sharedAccelBump = nil;
static NSString *apiKey_ = nil;

@interface AccelBump(){
    BumpEventBlock bumpEventBlock_;
    BumpMatchBlock bumpMatchBlock_;
    BumpChannelLockedBlock bumpChannelLockedBlock_;
    BumpDataReceivedBlock bumpDataReceivedBlock_;
    BumpConnectionStateChangedBlock bumpConnectionStateChangedBlock_;
    
    bump_vibration vibration_;
    
    //managers
    BumpManager *bumpManager;
    LocationManager *locationManager;
    NetworkManager *networkManager;
    UtilsManager *utilsManager;
    
}
@end

@implementation AccelBump

//return a singleton instance
+(AccelBump *)sharedInstance{
    if(sharedAccelBump == nil){
        sharedAccelBump = [[AccelBump alloc] init];
        [sharedAccelBump connect];
    }
    return sharedAccelBump;
}

+ (void)setApiKey:(NSString *)apiKey andUserName:(NSString *)userName{
    apiKey_  = apiKey;
}

- (void)connect{
    [self startListenersAndWaitForEvents];
}
- (void)disconnect{
    
}
- (void)setBumpEventBlock:(BumpEventBlock)bumpEventBlock{
    bumpEventBlock_ = bumpEventBlock;
}
- (void)setBumpMatchBlock:(BumpMatchBlock)bumpMatchBlock{
    bumpMatchBlock_ = bumpMatchBlock;
}
- (void)setBumpChannelLockedBlock:(BumpChannelLockedBlock)bumpChannelLockedBlock{
    bumpChannelLockedBlock_ = bumpChannelLockedBlock;
}
- (void)setBumpDataReceivedBlock:(BumpDataReceivedBlock)bumpDataReceivedBlock{
    bumpDataReceivedBlock_ = bumpDataReceivedBlock;
}
- (void)setBumpConnectionStateChangedBlock:(BumpConnectionStateChangedBlock)bumpConnectionStateChangedBlock{
    bumpConnectionStateChangedBlock_ = bumpConnectionStateChangedBlock;
}
- (void)setBumpVibration:(bump_vibration)vibration{
    vibration_ = vibration;
}

- (NSString *)userNameForChannel:(BumpChannel)channel{
    return @"";
}

#pragma mark - Core Methods
- (void)startListenersAndWaitForEvents{
    DataReceivedBlock dataReceivedBlock = ^(NSString *data){
        AccelBumpParser *parser = [[AccelBumpParser alloc] init];
        BumpMessage *message = [parser parse:data];
        switch (message.type) {
            case BUMP_MATCH:
                bumpMatchBlock_(message.channel);
                break;
            case BUMP_DATA:
                bumpDataReceivedBlock_([message.content dataUsingEncoding:NSUTF8StringEncoding],message.channel);
                break;
            case CONNECTED:
                bumpConnectionStateChangedBlock_(YES);
                break;
            case DISCONNECTED:
                bumpConnectionStateChangedBlock_(NO);
                break;
            default:
                break;
        }
    };
    
    utilsManager = [UtilsManager sharedInstance];
    [utilsManager setApiKey:apiKey_];
    networkManager = [NetworkManager sharedInstance];
    [networkManager connectToServer];
    [networkManager setDataReceivedBlock:dataReceivedBlock];
    locationManager = [LocationManager sharedInstance];
    bumpManager = [BumpManager sharedInstance];
    [bumpManager listenForAccelShakeWithHandler:bumpHandler];
    
}
#pragma mark - bump simulation methods
- (void)simulateBump{
    bumpHandler(YES);
}
#pragma mark - event handlers
BumpHandler bumpHandler = ^(BOOL bumpOccured){
        if(bumpOccured){
            [[LocationManager sharedInstance] start];
            CLLocation *location = [[LocationManager sharedInstance] getCurrentLocation];
            [[LocationManager sharedInstance] stop];
            NSString *messageToServer = [[UtilsManager sharedInstance] getBumpMessageToServerWithLocation:location];
            [[NetworkManager sharedInstance] writeToServer:messageToServer];
        }else{
            
        }
    };

#pragma mark - send data to server
- (void)sendData:(NSData *)data toChannel:(BumpChannel)channel{
    NSString *messageToServer = [utilsManager getBumpMessageWithContent:data forChannel:channel];
    [networkManager writeToServer:messageToServer];
}


@end
