//
//  NetworkManager.m
//  AccelBump
//
//  Created by manuMohan on 1/7/13.
//  Copyright (c) 2013 manuMohan. All rights reserved.
//

#import "NetworkManager.h"
#import "ABConstants.h"
static NetworkManager *sharedNetworkManager = nil;
@interface NetworkManager(){
    NSInputStream *inputStream;
    NSOutputStream *outputStream;
    
    DataReceivedBlock dataReceivedBlock_;
    
}

@end
@implementation NetworkManager
+ (NetworkManager *)sharedInstance{
    if(sharedNetworkManager == nil){
        sharedNetworkManager = [NetworkManager new];
    }
    return sharedNetworkManager;
}
- (void)setDataReceivedBlock:(DataReceivedBlock)dataReceivedBlock{
    dataReceivedBlock_ = dataReceivedBlock;
}
- (void)connectToServer{
    if(inputStream == nil && outputStream == nil){
        CFReadStreamRef readStream;
        CFWriteStreamRef writeStream;
        CFStreamCreatePairWithSocketToHost(kCFAllocatorDefault, (CFStringRef)serverIpAddress, (UInt32)serverPort, &readStream, &writeStream);
        CFReadStreamSetProperty(readStream, kCFStreamPropertyShouldCloseNativeSocket, kCFBooleanTrue);
        CFWriteStreamSetProperty(writeStream, kCFStreamPropertyShouldCloseNativeSocket, kCFBooleanTrue);
        inputStream = (__bridge NSInputStream *)readStream;
        outputStream = (__bridge NSOutputStream *)writeStream;
        
        [inputStream setDelegate:self];
        [outputStream setDelegate:self];
        [inputStream scheduleInRunLoop:[NSRunLoop currentRunLoop]
                               forMode:NSDefaultRunLoopMode];
        [outputStream scheduleInRunLoop:[NSRunLoop currentRunLoop]
                                forMode:NSDefaultRunLoopMode]; [inputStream open];
        [outputStream open];
    }else{
        
    }
}
- (void)disconnectServer{
    [inputStream close];
    [outputStream close];
    inputStream = nil;
    outputStream = nil;
}
- (void)writeToServer:(NSString *)dataString{
    NSData *data = [[NSData alloc] initWithData:[dataString dataUsingEncoding:NSUTF8StringEncoding]];
    if(outputStream != nil){
        [outputStream write:[data bytes] maxLength:[data length]];
    }
}
#pragma mark - nsstream delegate methods
- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode{
    switch (eventCode) {
        case NSStreamEventNone:
            break;
        case NSStreamEventOpenCompleted:
            break;
        case NSStreamEventEndEncountered:
            break;
        case NSStreamEventErrorOccurred:
            break;
        case NSStreamEventHasSpaceAvailable:
            break;
        case NSStreamEventHasBytesAvailable:
            if(aStream == inputStream){
                uint8_t buffer[1024];
                int len;
                while ([inputStream hasBytesAvailable]) {
                    len = [inputStream read:buffer maxLength:1024];
                    if(len > 0){
                        NSString *output = [[NSString alloc] initWithBytes:buffer length:len encoding:NSUTF8StringEncoding];
                        dataReceivedBlock_(output);
                    }
                }
            }
            break;
            
        default:
            break;
    }
}
@end
